package com.bocloud.sms.booter.controller;

import com.alibaba.druid.filter.config.ConfigTools;
import com.bocloud.cmp.boot.config.DataSourceConfiguration;
import com.bocloud.sms.entity.UpgradeHistory;
import com.bocloud.sms.entity.UpgradeTask;
import com.bocloud.sms.interfaces.UpgradeService;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.security.PublicKey;
import java.util.HashMap;
import java.util.Map;

/**
 * 系统升级
 *
 * @author luyanchao
 * @date 2021-06-09
 */
@RestController
@RequestMapping("/v1/upgrade")
@Tag(name = "系统升级")
@Slf4j
public class UpgradeController {


    @Autowired
    private UpgradeService upgradeService;
    @Autowired
    private DataSourceConfiguration dataSourceConfiguration;


    /**
     * 获取当前数据源信息
     *
     * @return url, user, pass
     */
    @Operation(summary = "获取当前数据源")
    @GetMapping("/datasource")
    public GeneralResult<Map<String, String>> getDatasource() {
        Map<String, String> result = new HashMap<>();
        result.put("url", dataSourceConfiguration.getUrl());
        result.put("user", dataSourceConfiguration.getUsername());
        PublicKey publicKey = ConfigTools.getPublicKey(dataSourceConfiguration.getPublicKey());
        try {
            String passPlainText = ConfigTools.decrypt(publicKey, dataSourceConfiguration.getPassword());
            result.put("pass", new AESEncryptor().encrypt(passPlainText, null));
        } catch (Exception e) {
            log.warn("MySQL密码解密失败：{}", e.getMessage());
            return new GeneralResult<>(true, result, "执行成功");
        }
        return new GeneralResult<>(true, result, "执行成功");
    }

    /**
     * 执行数据迁移
     *
     * @param param 参数
     */
    @Operation(summary = "执行数据迁移")
    @PostMapping("/datasource")
    public GeneralResult<Boolean> newMigration(@RequestBody Param param, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return upgradeService.newMigration(param.getUrl(), param.getUser(), param.getPass(), context);
    }

    /**
     * 重试数据迁移
     *
     * @param id 数据迁移id
     */
    @Operation(summary = "重试数据迁移")
    @PostMapping("/datasource/{id}")
    public GeneralResult<Boolean> retryMigration(@PathVariable(value = "id") Long id,
                                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return upgradeService.retryMigration(id, context);
    }

    /**
     * 查询数据迁移列表
     *
     * @param pager 分页参数
     */
    @Operation(summary = "查询数据迁移列表")
    @GetMapping("/records")
    public GeneralResult<GridBean<UpgradeHistory>> listUpgradeHistory(Pager pager) {
        return upgradeService.listUpgradeHistory(pager);
    }

    /**
     * 查询数据迁移详情
     *
     * @param id 数据迁移id
     */
    @Operation(summary = "查询数据迁移详情")
    @GetMapping("/records/{id}")
    public GeneralResult<UpgradeHistory> getUpgradeHistory(@PathVariable(value = "id") Long id) {
        return upgradeService.getUpgradeHistory(id);
    }


    /**
     * 查询数据迁移子任务列表
     *
     * @param id    数据迁移id
     * @param pager 分页参数
     */
    @Operation(summary = "查询数据迁移子任务列表")
    @GetMapping("/records/{id}/tasks")
    public GeneralResult<GridBean<UpgradeTask>> listUpgradeTask(Pager pager, @PathVariable(value = "id") Long id) {
        return upgradeService.listUpgradeTask(id, pager);
    }

    /**
     * 重试数据迁移子任务
     *
     * @param recordId 数据迁移id
     * @param taskId   子任务id
     */
    @Operation(summary = "重试数据迁移子任务")
    @PostMapping("/records/{recordId}/tasks/{taskId}")
    public GeneralResult<Boolean> retryUpgradeTask(@PathVariable(value = "recordId") Long recordId,
                                                   @PathVariable(value = "taskId") Long taskId,
                                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return upgradeService.retryUpgradeTask(taskId, context);
    }


    @Data
    @NoArgsConstructor
    public static class Param {
        private String url;
        private String user;
        private String pass;
    }

}
