package com.bocloud.sms.booter.scheduler;

import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cmp.boot.model.BocloudStatus;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.model.NotifyMessage;
import com.bocloud.sms.repository.TenantRepository;
import com.bocloud.sms.repository.UserRepository;
import com.megatron.common.model.RequestContext;
import com.megatron.framework.core.CurrentService;
import io.jsonwebtoken.lang.Collections;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 账号状态（过期）监控
 *
 * @author tyl
 * @time 2020/7/22 周三 9:55
 */
@Component
@Slf4j
public class AccountStatusMonitor implements InitializingBean {


    private final CurrentService service;
    private final UserRepository userRepository;
    private final TenantRepository tenantRepository;
    private final AmqpTemplate amqpTemplate;
    // 一天毫秒数
    private static final Long ONE_DAY = 24 * 60 * 60 * 1000L;

    public AccountStatusMonitor(CurrentService service, UserRepository userRepository, TenantRepository tenantRepository, AmqpTemplate amqpTemplate) {
        this.service = service;
        this.userRepository = userRepository;
        this.tenantRepository = tenantRepository;
        this.amqpTemplate = amqpTemplate;
    }

    @Scheduled(cron = "0 0 0 * * ?")
    private void accountStatusMonitor() throws Exception {
        log.info("用户有效期监控开启");
        log.info("当前服务是否是leader：{}", service.isLeader());
        if (!service.isLeader()) {
            return;
        }
        long threeDaysAgo = System.currentTimeMillis() - 3 * ONE_DAY;
        Date date = new Date(threeDaysAgo);
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        String format = simpleDateFormat.format(date);
        //管理员
        List<User> users = userRepository.listManagerHasExpiredDate(format);
        for (User user : users) {
            // 获取管理员的有效期时间
            Date expireDate = user.getExpiredDate();
            Long useTime = expireDate.getTime() - System.currentTimeMillis();
            // 剩余时间小于3天
            if (useTime <= 3 * ONE_DAY) {
                String content = null;
                if (useTime <= 0L) {
                    content = "用户账号已过期，请联系管理员进行重置！";
                    user.setStatus(BocloudStatus.Basic.EXPIRED.name());
                    user.setGmtModify(new Date());
                    user.setMenderId(1L);
                    userRepository.update(user);
                } else if (useTime % ONE_DAY == 0L) {
                    content = "用户账号还有【" + useTime / ONE_DAY + "天】将过期，请尽快修改或重置！";
                } else if (useTime % ONE_DAY > 0L) {
                    content = "用户账号还有【" + useTime / ONE_DAY + "天" + useTime % ONE_DAY / (60 * 60 * 1000L)
                            + "小时】将过期，请尽快修改或重置！";
                }

                NotifyMessage message = new NotifyMessage(new RequestContext(RequestContext.Catalog.Manager, user.getId(), 0L, null, null),
                        "用户账号使用时间到期预警", content, true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), "event.notice.queue",
                        message);
                log.info("账号到期预警信息发送完毕！管理员{}", user.getName());
            }
        }
        //租户
        List<Tenant> tenants = tenantRepository.listManagerHasExpiredDate();
        for (Tenant tenant : tenants) {
            Date expireDate = tenant.getExpiredDate();
            // 账号剩余使用时间
            Long useTime = expireDate.getTime() - System.currentTimeMillis();
            if (useTime <= 3 * ONE_DAY) {
                String content = null;
                if (useTime <= 0L) {
                    content = "租户账号已过期，请联系管理员进行重置！";
                    tenant.setStatus(BocloudStatus.Basic.EXPIRED.name());
                    tenant.setGmtModify(new Date());
                    tenant.setMenderId(1L);
                    tenantRepository.update(tenant);
                } else if (useTime % ONE_DAY == 0L) {
                    content = "租户账号还有【" + useTime / ONE_DAY + "天】将过期，请尽快修改或重置！";
                } else if (useTime % ONE_DAY > 0L) {
                    content = "租户账号还有【" + useTime / ONE_DAY + "天" + useTime % ONE_DAY / (60 * 60 * 1000L)
                            + "小时】将过期，请尽快修改或重置！";
                }
                NotifyMessage message = new NotifyMessage(new RequestContext(RequestContext.Catalog.Tenant, tenant.getId(), 0L, null, null),
                        "租户账号使用时间到期预警", content, true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), "event.notice.queue",
                        message);
                log.info("账号到期预警信息发送完毕！租户{}", tenant.getName());
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.accountStatusMonitor();
    }
}
