package com.bocloud.sms.booter.scheduler;

import com.alibaba.druid.util.StringUtils;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.model.NotifyMessage;
import com.bocloud.sms.repository.SystemConfigRepository;
import com.bocloud.sms.repository.TenantRepository;
import com.bocloud.sms.repository.UserRepository;
import com.megatron.common.model.RequestContext;
import com.megatron.framework.core.CurrentService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 用户密码使用时长监控
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/11
 */
@RequiredArgsConstructor
@Component
@Slf4j
public class UserPasswordMonitor implements InitializingBean {

    private final CurrentService service;
    private final TenantRepository tenantRepository;
    private final UserRepository userRepository;
    private final SystemConfigRepository systemConfigRepository;
    private final AmqpTemplate amqpTemplate;
    // 一天毫秒数
    private static final Long ONE_DAY = 24 * 60 * 60 * 1000L;

    /**
     * 每天执行一次
     *
     * @throws Exception
     */
    @Scheduled(cron = "0 0 0 * * ?")
    protected void monitor() throws Exception {
        log.info("当前服务是否是leader：{}", service.isLeader());
        if (!service.isLeader()) {
            return;
        }
        // 密码最大使用时长，若用户最后一次密码至今时长超过该值，则用户被冻结
        Long pwdMaxUsage = 90 * ONE_DAY;
        SystemConfig config = systemConfigRepository.queryByCode("pwdExpireDay");
        if (null != config) {
            String value = config.getValue();
            if (!StringUtils.isEmpty(value)) {
                pwdMaxUsage = Long.parseLong(value) * ONE_DAY;
                // 需对密码最大使用时间判断，如果为-1，表示过期时间为无期限
                if ("-1".equals(value)) {
                    return;
                }
            }
        }
        // 租户
        List<Tenant> tenants = tenantRepository.list(1, Integer.MAX_VALUE, new ArrayList<>(), null);
        for (Tenant tenant : tenants) {
            Date lastPwdModifyDate = tenant.getLastPwdModifyDate();
            if (null == lastPwdModifyDate) {
                tenant.setLastPwdModifyDate(new Date());
                tenant.setGmtModify(new Date());
                tenantRepository.update(tenant);
                continue;
            }
            // 密码已使用时间
            Long useTime = System.currentTimeMillis() - lastPwdModifyDate.getTime();
            // 密码剩余使用时间
            Long residueTime = pwdMaxUsage - useTime;
            if (residueTime <= 3 * ONE_DAY) {
                String content = null;
                if (residueTime < 0L) {
                    content = "登录密码已过期，租户账号将被冻结，请联系管理员进行重置密码！";
                    tenantRepository.lock(tenant.getId(), 1L);
                } else if (residueTime % ONE_DAY == 0L) {
                    content = "登录密码还有【" + residueTime / ONE_DAY + "天】将过期，请尽快修改或重置密码！";
                } else if (residueTime % ONE_DAY > 0L) {
                    content = "登录密码还有【" + residueTime / ONE_DAY + "天" + residueTime % ONE_DAY / (60 * 60 * 1000L)
                            + "小时】将过期，请尽快修改或重置密码！";
                }
                NotifyMessage message = new NotifyMessage(new RequestContext(RequestContext.Catalog.Tenant, tenant.getId(),
                        tenant.getId(), null, null), "租户账号密码使用时间到期预警", content, true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), "event.notice.queue",
                        message);
                log.info("密码到期预警信息发送完毕！租户{}", tenant.getName());
            }
        }
        // 用户
        List<User> users = userRepository.list(1, Integer.MAX_VALUE, new ArrayList<>(), null, null);
        for (User user : users) {
            if (1L == user.getId()) {
                // admin账户忽略，密码永不过期
                continue;
            }
            Date lastPwdModifyDate = user.getLastPwdModifyDate();
            if (null == lastPwdModifyDate) {
                user.setLastPwdModifyDate(new Date());
                user.setGmtModify(new Date());
                userRepository.update(user);
                continue;
            }
            // 密码已使用时间
            Long useTime = System.currentTimeMillis() - lastPwdModifyDate.getTime();
            // 密码剩余使用时间
            Long residueTime = pwdMaxUsage - useTime;
            if (residueTime <= 3 * ONE_DAY) {
                String content = null;
                if (residueTime < 0L) {
                    content = "登录密码已过期，用户将被冻结，请联系管理员进行重置密码！";
                    userRepository.lock(user.getId(), 1L);
                } else if (residueTime % ONE_DAY == 0L) {
                    content = "登录密码还有【" + residueTime / ONE_DAY + "天】将过期，请尽快修改或重置密码！";
                } else if (residueTime % ONE_DAY > 0L) {
                    content = "登录密码还有【" + residueTime / ONE_DAY + "天" + residueTime % ONE_DAY / (60 * 60 * 1000L)
                            + "小时】将过期，请尽快修改或重置密码！";
                }
                NotifyMessage userMessage = new NotifyMessage(new RequestContext(RequestContext.Catalog.User, user.getId(),
                        user.getTenantId(), null, null), "用户密码使用时间到期预警", content, true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), "event.notice.queue", userMessage);
                log.info("密码到期预警信息发送完毕！用户{}", user.getName());
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.monitor();
    }
}
