package com.bocloud.sms.booter.websocket.upload;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.booter.websocket.model.UploadACK;
import com.bocloud.sms.booter.websocket.model.UploadBean;
import com.bocloud.sms.booter.websocket.model.UploadCancel;
import com.bocloud.sms.booter.websocket.model.UploadMessage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.*;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.*;

@Component
@Slf4j
public class UploadServiceHandler implements WebSocketHandler {

    private static final String SAVE_SUCCESS = "SAVE_SUCCESS";
    private static final String SAVE_FAILURE = "SAVE_FAILURE";
    private static final String CANCEL_SUCCESS = "CANCEL_SUCCESS";
    private static final String CANCEL_FAILURE = "CANCEL_FAILURE";
    private static final String UPLOAD_CANCEL = "UPLOAD_CANCEL";
    private static List<WebSocketSession> currentUsers;

    static {
        UploadServiceHandler.setCurrentUsers(new ArrayList<>());
    }

    @Value("${spring.upload.path:/home/cmp}")
    private String path;

    /**
     * @return the currentUsers
     */
    public static List<WebSocketSession> getCurrentUsers() {
        return currentUsers;
    }

    /**
     * @param currentUsers the currentUsers to set
     */
    public static void setCurrentUsers(List<WebSocketSession> currentUsers) {
        UploadServiceHandler.currentUsers = currentUsers;
    }

    /**
     * 将二进制byte[]数组写入文件中
     *
     * @param byteBuffer byte[]数组
     * @param outputFile 文件位置
     * @return 成功: true 失败: false
     */
    private static boolean saveFile(ByteBuffer byteBuffer, String outputFile) {
        FileOutputStream fstream = null;
        File file = null;
        try {
            file = new File(outputFile);
            if (!file.exists() && !file.createNewFile()) {
                log.error("create new file error,path:{}", file.getPath());
                return false;
            }
            fstream = new FileOutputStream(file, true);
            fstream.write(byteBuffer.array());
        } catch (Exception e) {
            log.error("saveFile Exception:", e);
            return false;
        } finally {
            if (fstream != null) {
                try {
                    fstream.close();
                } catch (IOException e1) {
                    log.error("close fileimput Exception:", e1);
                }
            }
        }
        return true;
    }

    private String getPath() {
        if (this.path.endsWith(String.valueOf(File.separatorChar))) {
            return this.path.substring(0, this.path.length() - 1);
        }
        return path;
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        UploadServiceHandler.getCurrentUsers().add(session);
    }

    @Override
    public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
        if (message instanceof TextMessage) {
            handleText(session, (TextMessage) message);
        } else if (message instanceof BinaryMessage) {
            handleBinary(session, (BinaryMessage) message);
        } else {
            log.error("Unexpected WebSocket message type: " + message);
            throw new IllegalStateException("Unexpected WebSocket message type: " + message);
        }
    }

    /**
     * 处理字符串
     *
     * @param session
     * @param message
     */
    private void handleText(WebSocketSession session, TextMessage message) {
        UploadBean uploadBean = null;
        try {
            if (message.getPayload().contains("upload")) {
                uploadBean = JSONObject.parseObject(message.getPayload(), UploadBean.class);
                String date = DateFormatUtils.format(Calendar.getInstance(), "yyyyMMdd");
                String uuid = UUID.randomUUID().toString().replace("-", "");
                String path = File.separatorChar + "tmp" + File.separatorChar + date + File.separatorChar + uuid;
                File dir = new File(this.getPath(), path);
                if (!dir.exists()) {
                    boolean flag = dir.mkdirs();
                    if (!flag) {
                        log.error("mkdir failed, path: " + dir.getPath());
                        return;
                    }
                }
                path = path + File.separatorChar + uploadBean.getFilename();
                uploadBean.setTarget(this.getPath() + path);
                session.getAttributes().put("uploadBean", uploadBean);
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadACK(path))));
            } else if (message.getPayload().contains("sendover")) {
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage("TRUE"))));
            } else if (message.getPayload().contains(UPLOAD_CANCEL)) {
                log.warn("用户取消文件上传");
                uploadBean = (UploadBean) session.getAttributes().get("uploadBean");
                File file = new File(uploadBean.getTarget());
                this.deleteTmpFile(file);
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadCancel(CANCEL_SUCCESS))));
            }
        } catch (IOException e) {
            log.error("上传文件异常：", e);
            if (null == uploadBean) {
                return;
            }
            File file = new File(uploadBean.getTarget());
            this.deleteTmpFile(file);
        }
    }

    @SuppressWarnings("static-access")
    private void handleBinary(WebSocketSession session, BinaryMessage message) throws IOException {
        UploadBean uploadBean = (UploadBean) session.getAttributes().get("uploadBean");
        if (null == uploadBean) {
            return;
        }
        try {
            if (saveFile(message.getPayload(), uploadBean.getTarget())) {
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage(SAVE_SUCCESS))));
            } else {
                session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage(SAVE_FAILURE))));
                this.deleteTmpFile(new File(uploadBean.getTarget()));
            }
        } catch (IOException e) {
            log.error("上传文件异常：", e);
            session.sendMessage(new TextMessage(JSONObject.toJSONString(new UploadMessage(SAVE_FAILURE))));
            this.deleteTmpFile(new File(uploadBean.getTarget()));
        }
    }

    /**
     * 删除临时文件及文件夹
     *
     * @param file 文件
     */
    private void deleteTmpFile(File file) {
        if (file.exists()) {
            if (!file.delete()) {
                log.error("file delete failed, path:{} " + file.getPath());
                return;
            }
            File dirFile = file.getParentFile();
            if (!dirFile.exists()) {
                return;
            }
            if (Objects.requireNonNull(dirFile.list()).length == 0) {
                if (!dirFile.delete()) {
                    log.error("file delete failed, path:{} " + file.getPath());
                }
            }
        }
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus closeStatus) throws Exception {
        UploadServiceHandler.getCurrentUsers().remove(session);
    }

    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        if (session.isOpen()) {
            session.close();
        }
        UploadServiceHandler.getCurrentUsers().remove(session);
    }

    @Override
    public boolean supportsPartialMessages() {
        return false;
    }
}
