package com.bocloud.sms.service;

import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cmp.boot.model.BocloudStatus;
import com.bocloud.sms.entity.Message;
import com.bocloud.sms.interfaces.MessageService;
import com.bocloud.sms.repository.MessageRepository;
import com.megatron.common.model.*;
import com.megatron.common.model.OperateResult.OperateCategory;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author tyl
 */
@Slf4j
@Service("messageService")
public class MessageServiceImpl implements MessageService {

    private final MessageRepository messageRepository;
    private final LockFactory lockFactory;
    private final AmqpTemplate amqpTemplate;
    private static final String EXCHANGE = BoCloudExchange.Default.TOPIC.getExchange();
    private static final String BOCLOUD_MESSAGE_QUEUE = "bocloud.message.queue.#";

    public MessageServiceImpl(MessageRepository messageRepository, LockFactory lockFactory, AmqpTemplate amqpTemplate) {
        this.messageRepository = messageRepository;
        this.lockFactory = lockFactory;
        this.amqpTemplate = amqpTemplate;
    }

    @Override
    public GeneralResult<GridBean<Message>> list(Pager pager, RequestContext context) {
        GridBean<Message> gridBean;
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, Object> paramsMap = MapTools.simpleMap("ownerId", context.getTarget());
            params.add(new Param(paramsMap, Sign.EQ));
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            sorter.put("gmtCreate", Common.ONE);
            int total = this.messageRepository.count(params, context.getCatalog());
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.messageRepository.list(params, sorter, context.getCatalog());
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                List<Message> list =
                        this.messageRepository.list(pager.getPage(), pager.getRows(), params, sorter, context.getCatalog());
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query message list error:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 添加
     *
     * @param context
     * @param title
     * @param content
     * @return
     */
    @Override
    public Result create(RequestContext context, String title, String content) {
        Message message = new Message();
        try {
            if (null == context || null == context.getCatalog()) {
                context = new RequestContext(Catalog.Manager, 1L, null, null, null);
            }
            message.setSender(1L);
            message.setCreatorId(1L);
            message.setMenderId(1L);
            message.setGmtCreate(new Date());
            message.setName(title);
            message.setOwnerId(context.getTarget());
            message.setContent(content);
            message.setStatus(BocloudStatus.Message.UNREAD.name());
            if (context.getCatalog().equals(Catalog.Tenant)) {
                message.setCatalog(context.getCatalog());
            } else {
                message.setCatalog(Catalog.User);
            }
            messageRepository.save(message);
            OperateResult operateResult =
                    new OperateResult(true, content, "", null, OperateCategory.SiteMessage, context);
            amqpTemplate.convertAndSend(EXCHANGE, BOCLOUD_MESSAGE_QUEUE, operateResult);
        } catch (Exception e) {
            log.error("Create message error:", e);
            return new Result(false, "添加失败");
        }
        return new Result(true, "添加成功");
    }

    @Override
    public Result remove(Long id, Long userId) {
        String path = Message.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Message object = this.messageRepository.query(id);
            if (null == object) {
                log.warn("Message does not exist!");
                return new Result(false, "数据不存在");
            }
            this.messageRepository.remove(id, userId);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove message error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<Message> detail(Long id, RequestContext context) {
        try {
            Message message = this.messageRepository.query(id);
            if (null == message) {
                return new GeneralResult<>(false, "数据不存在");
            }
            if (message.getStatus().equals(BocloudStatus.Message.UNREAD.name())) {
                this.messageRepository.updateStatus(id, BocloudStatus.Message.READ.name(), context.getTarget());
                message.setStatus(BocloudStatus.Message.READ.name());
                OperateResult result =
                        new OperateResult(true, "状态变更成功", "message.change", null, OperateCategory.SiteMessage, context);
                this.amqpTemplate
                        .convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#", result);
            }
            return new GeneralResult<>(true, message, "查询成功");
        } catch (Exception e) {
            log.error("Get message error：", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result messageRead(RequestContext context) {
        String path = Message.class.getSimpleName() + "_" + context.getTarget();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            boolean messageRead = this.messageRepository.messageRead(context.getTarget(), context.getCatalog());
            if (messageRead) {
                OperateResult result =
                        new OperateResult(true, "状态变更成功", "message.change", null, OperateCategory.SiteMessage, context);
                this.amqpTemplate
                        .convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#", result);
            }
            return new Result(true, "标记成功");
        } catch (Exception e) {
            log.error("Modify Message fail:", e);
            return new Result(false, "标记失败");
        }
    }

    @Override
    public GeneralResult<Message> statistics(RequestContext context) {
        try {
            Message message = this.messageRepository.countStatus(context.getTarget(), context.getCatalog());
            return new GeneralResult<>(true, message, "查询成功");
        } catch (Exception e) {
            log.error("Modify Message status fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public void sendQueueMessage(String queueName, Map<String, Object> message) {
        this.amqpTemplate.convertAndSend(queueName, message);
    }

}
