package com.bocloud.sms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.ProjectService;
import com.bocloud.sms.model.ImpExpProjectExcelModel;
import com.bocloud.sms.model.ProjectModel;
import com.bocloud.sms.model.ProjectUserBean;
import com.bocloud.sms.repository.*;
import com.google.common.collect.Lists;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.bocloud.sms.service.utils.ExportUtil.*;

/**
 * @author tyl
 * @time 2020/6/3 周三 10:26
 */
@Service
@Slf4j
public class ProjectServiceImpl implements ProjectService {

    private final ProjectRepository projectRepository;
    private final UserRepository userRepository;
    private final UserProjectRepository userProjectRepository;
    private final BusinessRepository businessRepository;
    private final TenantRepository tenantRepository;
    private final LockFactory lockFactory;

    public ProjectServiceImpl(ProjectRepository projectRepository, UserRepository userRepository,
                              BusinessRepository businessRepository, TenantRepository tenantRepository, LockFactory lockFactory,
                              UserProjectRepository userProjectRepository) {
        this.projectRepository = projectRepository;
        this.userRepository = userRepository;
        this.businessRepository = businessRepository;
        this.tenantRepository = tenantRepository;
        this.lockFactory = lockFactory;
        this.userProjectRepository = userProjectRepository;
    }

    @Override
    public GeneralResult<GridBean<Project>> list(Pager pager, RequestContext requestContext, Boolean tenantId) {
        try {

            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            if (params.size() == 0 && tenantId) {
                params.add(new Param(MapTools.simpleMap("tenantId", 0), Sign.UEQ));
            }
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
            if (requestContext.getCatalog().equals(RequestContext.Catalog.Tenant)) {
                params.add(new Param(MapTools.simpleMap("tenantId", requestContext.getTenant()), Sign.EQ));
            } else if (requestContext.getCatalog().equals(RequestContext.Catalog.User)) {
                List<Project> projects = projectRepository.listUserManagerProjects(requestContext.getTarget());
                StringBuilder idBuilder = new StringBuilder();
                projects.forEach(project -> idBuilder.append(project.getId()).append(","));
                String ids;
                if (idBuilder.length() > 0) {
                    ids = idBuilder.substring(0, idBuilder.length() - 1);
                } else {
                    ids = "-1";
                }
                params.add(new Param(MapTools.simpleMap("id", ids), Sign.IN));
            }
            sorter.put("gmtCreate", Common.ONE);
            sorter.put("id", Common.ONE);
            int total = this.projectRepository.count(params);
            GridBean<Project> gridBean;
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.projectRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                List<Project> list = this.projectRepository.list(pager.getPage(), pager.getRows(), params, sorter);
                for (Project project : list) {
                    List<User> projectManagers = userRepository.getProjectManagers(project.getId());
                    StringBuilder builder = new StringBuilder();
                    for (User projectManager : projectManagers) {
                        builder.append(projectManager.getName()).append(",");
                    }
                    if (builder.length() > 0) {
                        project.setManagerName(builder.substring(0, builder.length() - 1));
                    }
                    int userCount = userRepository.count(null, Collections.singletonList(project.getId()), null);
                    project.setUserCount(userCount);
                }
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query notice icon list fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public GeneralResult condition(String condition, RequestContext requestContext) {
        try {
            JSONObject jsonObject = JSON.parseObject(condition);
            String conditionString = jsonObject.getString("condition");
            //查询自服务申请可筛选的项目列表
            if ("listApplyProjects".equals(conditionString)) {
                List<Project> projects;
                if (requestContext.getCatalog().equals(RequestContext.Catalog.Tenant)) {
                    projects = projectRepository.listTenantProject(requestContext.getTarget());
                } else if (requestContext.getCatalog().equals(RequestContext.Catalog.User)) {
                    projects = projectRepository.listUserProject(requestContext.getTarget());
                } else {
                    projects = projectRepository.listAll();
                }
                Long businessId = jsonObject.getLong("businessId");
                if (null != businessId && 0L != businessId) {
                    projects.removeIf(project -> !businessId.equals(project.getBusinessId()));
                }
                return new GeneralResult(true, projects, "查询成功");
            }
            if ("listUserManagerProjects".equals(conditionString)) {
                List<Project> projects = projectRepository.listUserManagerProjects(jsonObject.getLong("userId"));
                return new GeneralResult(true, projects, "查询成功");
            }
            if ("listUserVisibleProjects".equals(conditionString)) {
                List<Project> projects = projectRepository.listUserVisibleProjects(jsonObject.getLong("userId"));
                return new GeneralResult(true, projects, "查询成功");
            }
            if ("listTenantUserGlobalProjects".equals(conditionString)) {
                List<User> tenantUsers = userRepository.listByTenant(jsonObject.getLong("tenantId"));
                List<Long> userIds = tenantUsers.stream().map(User::getId).collect(Collectors.toList());
                List<UserProject> projects = userProjectRepository.listUsersGlobalProjects(userIds);
                return new GeneralResult(true, projects, "查询成功");
            }
            return new GeneralResult(false, "不支持该查询操作");
        } catch (Exception e) {
            log.error("list project by notice icon list fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result create(ProjectModel params, RequestContext requestContext) {
        try {
            Project project = new Project();
            BeanUtils.copyProperties(params, project);
            if (requestContext.getCatalog().equals(RequestContext.Catalog.Tenant)) {
                project.setTenantId(requestContext.getTenant());
            } else {
                project.setTenantId(0L);
            }
            if (projectRepository.exist(project.getName())) {
                return new Result(false, "项目名称已存在");
            }
            if (null != projectRepository.getByCode(project.getCode())) {
                return new Result(false, "该项目编码已存在");
            }
            Long tenantId = project.getTenantId();
            if (null != tenantId && 0L != tenantId) {
                Tenant tenant = tenantRepository.query(tenantId);
                Assert.notNull(tenant, "租户不存在");
            }
            //管理属性配置
            project.setVisibility(params.getVisibility().name());
            project.setCreatorId(requestContext.getTarget());
            this.projectRepository.save(project);
            List<UserProject> userProjects = new ArrayList<>();
            // 项目负责人配置
            //            Long managerId = project.getManagerId();
            //            if (null != managerId && 0L != managerId) {
            //                User managerUser = userRepository.query(managerId);
            //                if (null == managerUser) {
            //                    return new Result(false, "项目负责人不存在");
            //                }
            //                UserProject userProject = new UserProject(managerId, project.getId(), project.getTenantId(), true);
            //                userProjects.add(userProject);
            //            }
            //项目成员配置
            List<Long> userIds = Optional.ofNullable(params.getUserIds()).orElse(new ArrayList<>());
            for (Long userId : userIds) {
                UserProject userProject = new UserProject(null, userId, project.getId(), false);
                userProjects.add(userProject);
            }
            List<Long> managerIds = Optional.ofNullable(params.getManagerIds()).orElse(new ArrayList<>());
            for (Long userId : managerIds) {
                UserProject userProject = new UserProject(null, userId, project.getId(), true);
                userProjects.add(userProject);
            }
            if (!userProjects.isEmpty()) {
                for (UserProject userProject : userProjects) {
                    userProjectRepository.save(userProject);
                }
            }
            return new Result(true, "添加成功");
        } catch (Exception e) {
            log.error("Create project error:", e);
            return new Result(false, "添加失败:" + e.getMessage());
        }
    }

    @Override
    public Result modify(Long id, ProjectModel project, RequestContext requestContext) {
        String path = Project.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Project proj = this.projectRepository.query(id);
            if (null == proj) {
                log.warn("project does not exist!");
                return new Result(false, "数据不存在");
            }
            if (!proj.getName().equals(project.getName())) {
                if (projectRepository.exist(project.getName())) {
                    return new Result(false, "项目名称已存在");
                }
            }
            if (project.getCode() != null && !project.getCode().equals(proj.getCode())) {
                Project byCode = projectRepository.getByCode(project.getCode());
                if (byCode != null) {
                    return new Result(false, "项目编码已存在");
                }
                proj.setCode(project.getCode());
            }
            proj.setName(project.getName());
            proj.setRemark(project.getRemark());
            proj.setMenderId(requestContext.getTarget());
            proj.setGmtModify(new Date());
            proj.setTenantId(project.getTenantId());
            proj.setBusinessId(project.getBusinessId());
            proj.setVisibility(project.getVisibility().name());
            proj.setDepartId(project.getDepartId());
            proj.setDepartIds(project.getDepartIds());
            this.projectRepository.update(proj);
            return new Result(true, "修改成功");
        } catch (Exception e) {
            log.error("Update project fail:", e);
            return new Result(false, "修改失败:" + e.getMessage());
        }
    }

    @Override
    public Result remove(Long id, RequestContext requestContext) {
        String path = Project.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Project project = projectRepository.query(id);
            Assert.notNull(project, "项目信息不存在");
            this.projectRepository.remove(id, requestContext.getTarget());
            this.userProjectRepository.unBindProject(id, true);
            this.userProjectRepository.unBindProject(id, false);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove notice project fail:", e);
            return new Result(false, "删除失败:" + e.getMessage());
        }
    }

    @Override
    public Result batchRemove(List<Long> ids, RequestContext requestContext) {
        try {
            for (Long id : ids) {
                Project project = projectRepository.query(id);
                Assert.notNull(project, "项目信息不存在");
            }
            this.projectRepository.batchRemove(ids, requestContext.getTarget());
            this.userProjectRepository.unBindProject(ids);
            return new Result(true, "批量删除成功");
        } catch (Exception e) {
            log.error("batch remove notice project fail:", e);
            return new Result(false, "批量删除失败:" + e.getMessage());
        }
    }

    /**
     * 查询详细
     *
     * @param id
     * @return
     */
    @Override
    public GeneralResult<Project> detail(Long id) {
        try {
            Project project = projectRepository.query(id);
            List<User> projectManagers = userRepository.getProjectManagers(project.getId());
            StringBuilder builder = new StringBuilder();
            for (User projectManager : projectManagers) {
                builder.append(projectManager.getName()).append(",");
            }
            if (builder.length() > 0) {
                project.setManagerName(builder.substring(0, builder.length() - 1));
            }
            return new GeneralResult<>(true, project, "查询详情成功");
        } catch (Exception e) {
            log.error("query user detail fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result configUser(Long projectId, ProjectUserBean projectUserBean) {
        try {
            Project project = projectRepository.query(projectId);
            Assert.notNull(project, "项目信息不存在");
            // 删除旧成员记录
            userProjectRepository.unBindProject(projectId, true);
            userProjectRepository.unBindProject(projectId, false);
            ArrayList<UserProject> userProjects = new ArrayList<>();
            List<Long> managers = Optional.ofNullable(projectUserBean.getManagerIds()).orElse(new ArrayList<>());
            List<Long> users = Optional.ofNullable(projectUserBean.getUserIds()).orElse(new ArrayList<>());
            for (Long managerId : managers) {
                UserProject manager = new UserProject(null, managerId, projectId, true);
                userProjects.add(manager);
            }
            for (Long userId : users) {
                UserProject user = new UserProject(null, userId, projectId, false);
                userProjects.add(user);
            }
            if (!userProjects.isEmpty()) {
                for (UserProject userProject : userProjects) {
                    userProjectRepository.save(userProject);
                }
            }
            return new Result(true, "成员配置成功");
        } catch (Exception e) {
            log.error("User config fail:", e);
            return new Result(false, "用户配置失败:" + e.getMessage());
        }
    }

    @Override
    public Result configProjectManager(Long id, Long userId) {
        try {
            UserProject userProject = userProjectRepository.getUserProject(userId, id);
            Assert.notNull(userProject, "该用户不属于该项目成员");
            userProject.setProjectAdmin(true);
            userProjectRepository.update(userProject);
            return new Result(true, "设置成功");
        } catch (Exception e) {
            log.error("User config project manager fail:", e);
            return new Result(false, "用户配置失败:" + e.getMessage());
        }
    }

    /**
     * 根据查询条件导出项目列表
     *
     * @param params
     * @param requestContext
     * @return
     */
    @Override
    public XSSFWorkbook export(List<Param> params, RequestContext requestContext) {
        try {
            List<Project> projects = Lists.newArrayList();
            List<Long> projectIds = Lists.newArrayList();
            if (null != requestContext.getTenant() && 0L != requestContext.getTenant()) {
                if (requestContext.getCatalog().equals(RequestContext.Catalog.Tenant)) {
                    // 租户下的所有项目
                    List<Project> tenantProjects = projectRepository.listTenantProject(requestContext.getTenant());
                    Map<Long, Project> projectMap = new HashMap<>(tenantProjects.size());
                    for (Project project : tenantProjects) {
                        projectMap.put(project.getId(), project);
                    }
                    projects.addAll(projectMap.values());
                } else {
                    projects = projectRepository.listUserProject(requestContext.getTarget());
                }
                // 解决 目前租户如果没有项目可以看到全部项目，所以需要过滤 2019年9月16日16:44:34
                if (projects.size() == 0) {
                    Project project = new Project();
                    project.setId(-1L);
                    projects.add(project);
                }
            }
            projects.forEach(project -> projectIds.add(project.getId()));
            List<Project> result = projectRepository.export(params, projectIds);
            params.clear();
            for (Project proj : result) {
                List<User> projectManagers = userRepository.getProjectManagers(proj.getId());
                StringBuilder builder = new StringBuilder();
                for (User projectManager : projectManagers) {
                    builder.append(projectManager.getName()).append(",");
                }
                if (builder.length() > 0) {
                    proj.setManagerName(builder.substring(0, builder.length() - 1));
                }
                int userCount = userRepository.count(params, Collections.singletonList(proj.getId()), null);
                proj.setUserCount(userCount);
            }
            // 创建HSSFWorkbook
            String sheetName = "项目统计数据";
            String[] cellTitle = new String[]{"项目名称", "所属业务", "所属租户", "项目描述"};
            String[][] rows;
            if (!ListTool.isEmpty(result)) {
                rows = new String[result.size()][cellTitle.length];
                for (int i = 0; i < result.size(); i++) {
                    Project project = result.get(i);
                    rows[i][0] = project.getName();
                    rows[i][1] = project.getBusinessName();
                    rows[i][2] = project.getTenantName();
                    rows[i][3] = project.getRemark();
                }
            } else {
                rows = new String[0][cellTitle.length];
            }
            return getXssfWorkbook(sheetName, cellTitle, rows);
        } catch (Exception e) {
            log.error("create project work book fail:", e);
            return null;
        }
    }

    @Override
    public GeneralResult<Business> getBusinessDetail(Long id, RequestContext requestContext) {
        try {
            Project project = projectRepository.query(id);
            if (project.getBusinessId() != null) {
                Business business = businessRepository.query(project.getBusinessId());
                return new GeneralResult<>(true, business, "查询详情成功");
            } else {
                return new GeneralResult<>(true, null, "查询详情成功");
            }
        } catch (Exception e) {
            log.error("query Business detail fail:", e);
            return new GeneralResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public XSSFWorkbook importTemp2(String name, RequestContext requestContext) {
        //创建一个HSSFWorkbook，对应一个Excel文件
        try (XSSFWorkbook wb = new XSSFWorkbook()) {
            List<Tenant> tenants;
            if (requestContext.getCatalog().equals(RequestContext.Catalog.Manager)) {
                tenants = tenantRepository.list();
            } else {
                tenants = Lists.newArrayList(tenantRepository.query(requestContext.getTenant()));
            }
            List<Business> businesses = businessRepository.list(1, Integer.MAX_VALUE, null, null);
            //在workbook中添加一个sheet,对应Excel文件中的主sheet
            XSSFSheet sheet = wb.createSheet(name);
            //在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制
            XSSFRow row = sheet.createRow(0);
            //创建单元格，并设置值表头 设置表头居中
            XSSFCellStyle style = wb.createCellStyle();
            // 创建一个居中格式
            style.setAlignment(HorizontalAlignment.CENTER);
            // 设置为文本格式，防止身份证号变成科学计数法
            DataFormat format = wb.createDataFormat();
            style.setDataFormat(format.getFormat("@"));
            String[] title = {"项目名称", "项目编码", "所属业务", "所属租户", "项目描述"};
            for (int i = 0; i < title.length; i++) {
                XSSFCell cell = row.createCell(i);
                cell.setCellValue(title[i]);
                cell.setCellStyle(style);
                sheet.setColumnWidth(i, 20 * 256);
                sheet.setDefaultColumnStyle(i, style);
            }
            List<String> businessNames = businesses.stream().map(Business::getName).collect(Collectors.toList());
            if (!businessNames.isEmpty()) {
                addValidation(sheet, businessNames, 2);
            }
            List<String> tenantNames = tenants.stream().map(Tenant::getName).limit(50).collect(Collectors.toList());
            XSSFSheet dateSheet = wb.createSheet("数据源");
            for (int i = 0, tenantNamesSize = tenantNames.size(); i < tenantNamesSize; i++) {
                String tenantName = tenantNames.get(i);
                Row dataRow = dateSheet.createRow(i);
                Cell tenantCell = dataRow.createCell(0);
                tenantCell.setCellValue(tenantName);
            }
            XSSFDataValidationHelper tenantValidationHelper = new XSSFDataValidationHelper(sheet);
            DataValidationConstraint tenantValidationHelperFormulaListConstraint =
                    tenantValidationHelper.createFormulaListConstraint("数据源!$A$1:$A$" + tenants.size());
            CellRangeAddressList tenantList = new CellRangeAddressList(1, 50000, 3, 3);
            //数据有效性对象
            DataValidation tenantValidationHelperValidation =
                    tenantValidationHelper.createValidation(tenantValidationHelperFormulaListConstraint, tenantList);
            //设置输入错误提示信息
            tenantValidationHelperValidation.createErrorBox("非法输入", "不允许输入，请选取下拉值！");
            sheet.addValidationData(tenantValidationHelperValidation);
            wb.setSheetHidden(wb.getSheetIndex("数据源"), true);
            return wb;
        } catch (Exception e) {
            log.error("get import template error", e);
            return null;
        }
    }

    @Override
    public Result importTemp(HttpServletResponse response) {
        boolean exportExcel = exportExcel("项目列表" + System.currentTimeMillis(), "项目列表", response, new ArrayList<>(), ImpExpProjectExcelModel.class, null);
        return exportExcel ? new Result(true, "模板下载成功") : new Result(false, "模板下载失败");
    }

    @Override
    public Result importData(MultipartFile multipartFile, RequestContext requestContext) {
        try {
            String originalFilename = multipartFile.getOriginalFilename();
            Assert.notNull(originalFilename, "文件名不能为空");
            InputStream inputStream = multipartFile.getInputStream();
            Workbook wb = new XSSFWorkbook(inputStream);
            List<Project> projects = readExcel(wb, requestContext);
            Assert.notEmpty(projects, "导入模板数据为空");
            Map<String, Tenant> tenantMap = tenantRepository.list().stream()
                    .collect(Collectors.toMap(Tenant::getName, Function.identity(), (v1, v2) -> v1));
            Map<String, Business> businessMap = businessRepository.list(1, Integer.MAX_VALUE, null, null).stream()
                    .collect(Collectors.toMap(Business::getName, Function.identity(), (v1, v2) -> v1));
            Map<String, Project> projectMap = projectRepository.listAll().stream()
                    .collect(Collectors.toMap(Project::getName, Function.identity(), (v1, v2) -> v1));
            // 错误数量统计
            int errorNumber = 0;
            ListIterator<Project> listIterator = projects.listIterator();
            while (listIterator.hasNext()) {
                Project project = listIterator.next();
                String name = project.getName();
                Project existProject = projectMap.get(name);
                if (existProject != null) {
                    log.warn("project " + project.getName() + "exist!");
                    errorNumber++;
                    listIterator.remove();
                    continue;
                }
                String businessName = project.getBusinessName();
                if (StringUtils.hasText(businessName)) {
                    Business business = businessMap.get(businessName);
                    if (business == null) {
                        log.warn("project " + name + "'s business " + businessName + " don't exist!");
                        errorNumber++;
                        listIterator.remove();
                        continue;
                    } else {
                        project.setBusinessId(business.getId());
                    }
                }
                String tenantName = project.getTenantName();
                if (StringUtils.hasText(tenantName)) {
                    Tenant tenant = tenantMap.get(tenantName);
                    if (tenant == null) {
                        log.warn("project " + name + "'s tenant " + tenantName + " don't exist!");
                        errorNumber++;
                        listIterator.remove();
                    } else {
                        project.setTenantId(tenant.getId());
                    }
                } else {
                    if (requestContext.getTenant() != null) {
                        project.setTenantId(requestContext.getTenant());
                    } else {
                        project.setTenantId(0L);
                    }
                }
            }
            if (!projects.isEmpty()) {
                projectRepository.batchSave(projects, projects.size());
            }
            if (errorNumber == 0) {
                return new Result(true, "导入成功");
            } else {
                return new Result(true, projects.size() + "条数据导入成功," + errorNumber + "条数据导入失败");
            }
        } catch (Exception e) {
            log.error("import project data error,", e);
            return new Result(false, "导入项目信息失败：" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<List<Project>> queryByDepart(String departName) {
        return new GeneralResult(true, projectRepository.queryByDepartName(departName), "查询成功");
    }

    @Override
    public GeneralResult<List<Project>> queryByDepartIds(Pager pager, List<Long> departIds) {
        Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(8, 0.75F));
        sorter.put("gmtCreate", Common.ONE);
        sorter.put("id", Common.ONE);
        return new GeneralResult(true, projectRepository.queryByDepartIds(pager.getPage(), pager.getRows(), departIds, sorter), "查询成功");
    }

    @Override
    public GeneralResult<Project> getByProjectName(String projectName) {
        return new GeneralResult(true, projectRepository.getByProjectName(projectName), "查询成功");
    }

    private List<Project> readExcel(Workbook wb, RequestContext context) {
        Sheet sheet = wb.getSheetAt(0);
        List<Project> projects = new ArrayList<>();
        for (int rowNum = 1; rowNum <= sheet.getLastRowNum(); rowNum++) {
            Row row = sheet.getRow(rowNum);
            Project project = new Project();
            Cell name = row.getCell(0);
            if (name != null) {
                name.setCellType(CellType.STRING);
                project.setName(name.getStringCellValue().trim());
            }
            Cell code = row.getCell(1);
            if (code != null) {
                code.setCellType(CellType.STRING);
                String codeValue = code.getStringCellValue();
                project.setCode(codeValue);
            }
            project.setVisibility(ProjectModel.Visibility.manageable.name());
            Cell businessName = row.getCell(2);
            if (businessName != null) {
                businessName.setCellType(CellType.STRING);
                project.setBusinessName(businessName.getStringCellValue().trim());
            }
            Cell tenant = row.getCell(3);
            if (tenant != null) {
                tenant.setCellType(CellType.STRING);
                String tenantName = tenant.getStringCellValue();
                project.setTenantName(tenantName);
            }
            Cell remark = row.getCell(4);
            if (remark != null) {
                remark.setCellType(CellType.STRING);
                String remarkValue = remark.getStringCellValue();
                project.setRemark(remarkValue);
            }
            project.setCreatorId(context.getTarget());
            projects.add(project);
        }
        return projects;
    }
}

