package com.bocloud.sms.service.utils;

import com.alibaba.fastjson.JSONObject;
import com.megatron.framework.registry.RegistryClient;
import com.bocloud.sms.entity.SystemConfig;
import com.bocloud.sms.repository.SystemConfigRepository;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.curator.framework.recipes.cache.NodeCache;
import org.apache.zookeeper.CreateMode;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

/**
 * 云服务供应商注册器
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年9月22日
 */
@Component
@Slf4j
public class ConfigRegister implements InitializingBean {

    private static NodeCache nodeCache;
    private final SystemConfigRepository configRepository;
    private final RegistryClient registryClient;

    @Autowired
    public ConfigRegister(SystemConfigRepository configRepository, RegistryClient registryClient) {
        this.configRepository = configRepository;
        this.registryClient = registryClient;
    }

    public void updateConfig(List<SystemConfig> configs) throws Exception {
        Map<String, Object> map = convertConfigs(configs);
        String data = JSONObject.parseObject(JSONObject.toJSONString(map)).toString();
        this.registryClient.getClient().createAndSet(this.registryClient.getService().getSystem(), data, CreateMode.PERSISTENT);
    }

    /**
     * 服务启动时将数据库中的云服务供应商信息注册到ZooKeeper中
     *
     * @throws Exception 运行异常
     */
    public void start() throws Exception {
        this.registryClient.getClient().createIfAbsent(this.registryClient.getService().getSystem(), null, CreateMode.PERSISTENT);
        List<SystemConfig> configs = configRepository.listAll();
        Map<String, Object> map = convertConfigs(configs);
        String data = JSONObject.parseObject(JSONObject.toJSONString(map)).toString();
        this.registryClient.getClient().setData(this.registryClient.getService().getSystem(), data.getBytes());
        Configuration.getInstance().update(data);
        // 处理ZooKeeper上的系统配置发生变化时的处理逻辑：更新内部的缓存
        nodeCache.getListenable().addListener(() -> {
            log.info("system config data changed,Refresh them in cache!");

            Configuration.getInstance().update(new String(nodeCache.getCurrentData().getData(), StandardCharsets.UTF_8));
            log.info("Refresh system config Success!");
        });
    }

    private Map<String, Object> convertConfigs(List<SystemConfig> configs) {
        Map<String, Object> map = Maps.newHashMap();
        for (SystemConfig config : configs) {
            map.put(config.getCode(), config.getValue());
        }
        return map;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        nodeCache = new NodeCache(this.registryClient.getClient().getClient(), this.registryClient.getService().getSystem());
        nodeCache.start();
        this.start();
    }
}
