package com.bocloud.sms.repository;

import com.bocloud.sms.entity.Project;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.framework.core.CurrentService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/6/2 周二 15:57
 */
@Repository("projectRepository")
public class ProjectRepository extends JdbcGenericDao<Project, Long> {

    private static final Logger logger = LoggerFactory.getLogger(ProjectRepository.class);

    public ProjectRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Project> list(int page, int rows, List<Param> params, Map<String, String> sorter) throws DataBaseException {
        String sql = "select a.*,b.name tenant_name,c.name business_name, d.name as depart_name from project a " +
                "left join tenant b on a.tenant_id = b.id " +
                "left join business c on c.id = a.business_id " +
                "left join department d on a.depart_id = d.id " +
                "where a.is_deleted = 0";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(Project.class, sql, paramMap);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws DataBaseException {
        String sql = "select a.id,a.name from project a where is_deleted = 0 ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        return this.list(Project.class, sql, paramMap).stream().map(project -> new SimpleBean(project.getId(), project.getName())).collect(Collectors.toList());
    }

    public Project query(Long id) throws DataBaseException {
        String sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.id = :id and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Project> list = this.list(Project.class, sql, params);
        Assert.isTrue(list.size() <= 1, "project more than one in table");
        return list.isEmpty() ? null : list.get(0);
    }

    public List<Project> listAll() throws DataBaseException {
        String sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.is_deleted = 0";
        return this.list(Project.class, sql);
    }

    public boolean remove(Long id, Long userId) throws DataBaseException {
        String sql = "update project set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public boolean batchRemove(List<Long> ids, Long userId) throws DataBaseException {
        String sql = "update project set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id in (:ids) ";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    public int count(List<Param> params) throws DataBaseException {
        String sql = "select count(1) from project where is_deleted = 0";
        Map<String, Object> param = getQueryBuilder().getParam(params);
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, param).intValue();
    }

    public List<Project> listUserProject(Long userId) throws DataBaseException {
        String sql = "select distinct a.* from project a left join user_project b on a.id = b.project_id where a.is_deleted = 0 and b.user_id = :userId";
        Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
        return this.list(Project.class, sql, paramMap);
    }

    public List<Project> listUserManagerProjects(Long userId) throws DataBaseException {
        String sql = "select distinct a.* from project a left join user_project b on a.id = b.project_id where a.is_deleted = 0 and b.user_id = :userId and b.project_admin = 1";
        Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
        return this.list(Project.class, sql, paramMap);
    }

    public List<Project> listUserVisibleProjects(Long userId) throws DataBaseException {
        String sql = "select distinct a.* from project a left join user_project b on a.id = b.project_id where a.is_deleted = 0 and b.user_id = :userId and a.visibility != 'invisible'  ";
        Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
        return this.list(Project.class, sql, paramMap);
    }

    public List<Project> listTenantProject(Long tenantId) throws DataBaseException {
        String sql = "select a.* from project a where a.is_deleted = 0 and a.tenant_id = :tenantId";
        Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
        return this.list(Project.class, sql, paramMap);
    }

    public boolean exist(String name) throws DataBaseException {
        String sql = "select a.* from project a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<Project> result = this.list(Project.class, sql, paramMap);
        return !result.isEmpty();
    }

    public List<Project> businessProjects(Long businessId) throws DataBaseException {
        String sql = "select * from project where business_id =:businessId and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("businessId", businessId);
        return this.list(Project.class, sql, params);

    }

    /**
     * 查询项目列表
     *
     * @param tenantId
     * @param visibility
     * @return
     * @throws DataBaseException
     */
    public List<Project> list(Long tenantId, String visibility) throws DataBaseException {
        String sql = "select * from project where is_deleted = 0 and tenant_id =:tenantId and visibility = :visibility";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("visibility", visibility);
        return this.list(Project.class, sql, params);
    }

    /**
     * 导出项目列表
     *
     * @param params
     * @param projectIds
     * @return
     * @throws DataBaseException
     */
    public List<Project> export(List<Param> params, List<Long> projectIds) throws DataBaseException {
        String sql = "select a.*,b.name tenant_name,c.name business_name from project a left join tenant b on a.tenant_id = b.id left join business c on c.id = a.business_id where a.is_deleted = 0";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        if (!ListTool.isEmpty(projectIds)) {
            sql += " and a.id in (:projectIds)";
            paramMap.put("projectIds", projectIds);
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, null), "a");
        return this.list(Project.class, sql, paramMap);
    }

    /**
     * 根据业务ID查询
     *
     * @param businessId
     * @return
     * @throws DataBaseException
     */
    public List<Project> listByBusiness(Long businessId) throws DataBaseException {
        String sql = "select * from project where is_deleted = 0 and business_id =:businessId";
        Map<String, Object> params = MapTools.simpleMap("businessId", businessId);
        return this.list(Project.class, sql, params);
    }

    /**
     * 根据业务id查询
     *
     * @param businessId
     * @param tenantId
     * @return
     * @throws DataBaseException
     */
    public List<Project> listByBusiness(Long businessId, Long tenantId) throws DataBaseException {
        String sql = "select * from project where is_deleted = 0 and business_id =:businessId and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("businessId", businessId);
        params.put("tenantId", tenantId);
        return this.list(Project.class, sql, params);
    }


    /**
     * 根据项目id查询
     *
     * @param projectIds
     * @return
     * @throws DataBaseException
     */
    public List<Project> listByIds(List<Long> projectIds) throws DataBaseException {
        String sql = "select * from project where is_deleted = 0 and id in (:projectIds)";
        Map<String, Object> params = MapTools.simpleMap("projectIds", projectIds);
        return this.list(Project.class, sql, params);
    }

    /**
     * 根据编码查询
     *
     * @param code
     * @return
     * @throws DataBaseException
     */
    public Project getByCode(String code) throws DataBaseException {
        String sql = "select a.* from project a where a.code = :code  and a.is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<Project> list = this.list(Project.class, sql, params);
        Assert.isTrue(list.size() <= 1, "project more than one in table");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 获取没有配置业务的项目
     *
     * @return
     * @throws DataBaseException
     */
    public List<Project> getNoBusinessProjects() throws DataBaseException {
        String sql = "select  * from project where is_deleted = 0 and business_id is null ";
        return this.list(Project.class, sql);
    }

    public List<Project> queryByDepartName(String departName) throws DataBaseException {
        String sql = "select * from project where is_deleted = 0 and depart_id = (select id from department where name = '" + departName + "')";
        return this.list(Project.class, sql);
    }

    public List<Project> queryByDepartIds(int page, int rows, List<Long> departIds, Map<String, String> sorter) throws DataBaseException {
        String sql = "select p.name, d.name as depart_name" +
                " from project p" +
                " left join department d on p.depart_id = d.id" +
                " where d.id in (:departIds)";
        Map<String, Object> params = MapTools.simpleMap("departIds", departIds);
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, null, sorter), "p");
        return this.list(Project.class, sql, params);
    }

    public Project getByProjectName(String name) throws DataBaseException {
        String sql = "select a.* from project a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        List<Project> list = this.list(Project.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }
}

